@@ -2,7 +2,7 @@
 # -*- coding: utf8 -*-
 
 # ============================================================================
-#  Copyright (c) 2014-2019 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -14,11 +14,6 @@
 #  limitations under the License.
 # ============================================================================
 
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
-from collections import OrderedDict
 import unittest
 
 from testing_utils import get_temp_dir
@@ -33,6 +28,7 @@ from unittest.case import skip
 
 
 class GenTest(unittest.TestCase):
+
     def test_check_duplicated_columns(self):
         test_file = get_test_loc('test_gen/dup_keys.csv')
         expected = [Error(ERROR, 'Duplicated column name(s): copyright with copyright\nPlease correct the input and re-run.')]
@@ -46,40 +42,46 @@ class GenTest(unittest.TestCase):
         assert expected == result
 
     def test_check_duplicated_about_resource(self):
-        test_dict = [
-            {'about_resource': '/test/test.c', 'version': '1.03', 'name': 'test.c'},
-            {'about_resource': '/test/abc/', 'version': '1.0', 'name': 'abc'},
-            {'about_resource': '/test/test.c', 'version': '1.04', 'name': 'test1.c'}]
-        expected = [
-            Error(CRITICAL,
-                  "The input has duplicated values in 'about_resource' field: /test/test.c")]
-        result = gen.check_duplicated_about_resource(test_dict)
-        assert expected == result
+        arp_list = ['/test/test.c', 'test/test1.h']
+        arp1 = '/test/test.c'
+        arp2 = '/test/tmp/test.c'
+        expected = Error(CRITICAL,
+                  "The input has duplicated values in 'about_resource' field: " + arp1)
+        result1 = gen.check_duplicated_about_resource(arp1, arp_list)
+        result2 = gen.check_duplicated_about_resource(arp2, arp_list)
+        assert result1 == expected
+        assert result2 == ''
 
     def test_check_newline_in_file_field(self):
-        test_dict = [
-            {'about_resource': '/test/test.c', 'name': 'test.c', 'notice_file': 'NOTICE\nNOTICE2'},
-            {'about_resource': '/test/abc/', 'version': '1.0', 'name': 'abc'},
-            {'about_resource': '/test/test.c', 'version': '1.04', 'name': 'test1.c'}]
+        test_dict1 = {'about_resource': '/test/test.c', 'name': 'test.c', 'notice_file': 'NOTICE\nNOTICE2'}
+        test_dict2 = {'about_resource': '/test/test.c', 'name': 'test.c', 'notice_file': 'NOTICE, NOTICE2'}
         expected = [
             Error(CRITICAL,
                   "New line character detected in 'notice_file' for '/test/test.c' which is not supported."
                   "\nPlease use ',' to declare multiple files.")]
-        result = gen.check_newline_in_file_field(test_dict)
-        assert expected == result
+        result1 = gen.check_newline_in_file_field(test_dict1)
+        result2 = gen.check_newline_in_file_field(test_dict2)
+        assert result1 == expected
+        assert result2 == []
+
+    def test_check_about_resource_filename(self):
+        arp1 = '/test/t@est.c'
+        arp2 = '/test/t!est.c'
+        msg = ("Invalid characters present in 'about_resource' "
+                   "field: " + arp2)
+        expected2 = Error(ERROR, msg)
+        result1 = gen.check_about_resource_filename(arp1)
+        result2 = gen.check_about_resource_filename(arp2)
+        assert result1 == ''
+        assert result2 == expected2
 
     def test_load_inventory(self):
         location = get_test_loc('test_gen/inv.csv')
         base_dir = get_temp_dir()
-        errors, abouts = gen.load_inventory(location, base_dir)
+        errors, abouts = gen.load_inventory(location, base_dir=base_dir)
 
-        expected_errors = [
-            Error(INFO, 'Field custom1 is a custom field.'),
-            Error(INFO, 'Field about_resource: Path')
-        ]
-        for exp, err in zip(expected_errors, errors):
-            assert exp.severity == err.severity
-            assert err.message.startswith(exp.message)
+        expected_num_errors = 29
+        assert len(errors) == expected_num_errors 
 
         expected = (
 '''about_resource: .
@@ -91,6 +93,35 @@ description: |
 custom1: |
   multi
   line
+'''
+        )
+        result = [a.dumps() for a in abouts]
+        assert expected == result[0]
+
+    def test_load_inventory_without_about_resource(self):
+        location = get_test_loc('test_gen/inv_no_about_resource.csv')
+        base_dir = get_temp_dir()
+        from_attrib = False
+        errors, abouts = gen.load_inventory(location, base_dir=base_dir, from_attrib=from_attrib)
+        expected_error = [Error(CRITICAL,  "The essential field 'about_resource' is not found in the <input>")]
+
+        assert errors == expected_error
+        assert abouts == [] 
+
+    def test_load_inventory_without_about_resource_from_attrib(self):
+        location = get_test_loc('test_gen/inv_no_about_resource.csv')
+        base_dir = get_temp_dir()
+        from_attrib = True
+        errors, abouts = gen.load_inventory(location, base_dir=base_dir, from_attrib=from_attrib)
+
+        expected_num_errors = 0
+        assert len(errors) == expected_num_errors 
+
+        expected = (
+'''about_resource: .
+name: AboutCode
+version: 0.11.0
+license_expression: apache-2.0
 '''
         )
         result = [a.dumps() for a in abouts]
@@ -99,15 +130,13 @@ custom1: |
     def test_load_inventory_with_errors(self):
         location = get_test_loc('test_gen/inv4.csv')
         base_dir = get_temp_dir()
-        errors, abouts = gen.load_inventory(location, base_dir)
-
+        errors, abouts = gen.load_inventory(location, base_dir=base_dir)
         expected_errors = [
-            Error(CRITICAL, "Field name: 'confirmed copyright' contains illegal name characters: 0 to 9, a to z, A to Z and _."),
-            Error(INFO, 'Field resource is a custom field.'),
-            Error(INFO, 'Field test is a custom field.'),
-            Error(INFO, 'Field about_resource: Path')
+            Error(ERROR, "Field name: ['confirmed copyright'] contains illegal name characters (or empty spaces) and is ignored."),
+            Error(INFO, 'Field about_resource: Path'),
+            Error(INFO, "Field ['resource', 'test'] is a custom field.")
         ]
-        # assert [] == errors
+
         for exp, err in zip(expected_errors, errors):
             assert exp.severity == err.severity
             assert err.message.startswith(exp.message)
@@ -126,6 +155,43 @@ custom1: |
         result = [a.dumps() for a in abouts]
         assert expected == result[0]
 
+    def test_load_inventory_simple_xlsx(self):
+        location = get_test_loc('test_gen/load/simple_sample.xlsx')
+        base_dir = get_temp_dir()
+        errors, abouts = gen.load_inventory(location, base_dir=base_dir)
+        expected_errors = []
+        result = [(level, e) for level, e in errors if level > INFO]
+        assert expected_errors == result
+
+        assert abouts[0].name.value == 'cryptohash-sha256'
+        assert abouts[1].name.value == 'some_component'
+        
+        assert abouts[0].version.value == 'v 0.11.100.1'
+        assert abouts[1].version.value == 'v 0.0.1'
+
+        assert abouts[0].license_expression.value == 'bsd-new and mit'
+        assert abouts[1].license_expression.value == 'mit'
+
+
+    def test_load_scancode_json(self):
+        location = get_test_loc('test_gen/load/clean-text-0.3.0-lceupi.json')
+        inventory = gen.load_scancode_json(location)
+
+        expected = {'about_resource': 'clean-text-0.3.0', 'type': 'directory',
+                    'name': 'clean-text-0.3.0', 'base_name': 'clean-text-0.3.0',
+                    'extension': '', 'size': 0, 'date': None, 'sha1': None,
+                    'md5': None, 'sha256': None, 'mime_type': None, 'file_type': None,
+                    'programming_language': None, 'is_binary': False, 'is_text': False,
+                    'is_archive': False, 'is_media': False, 'is_source': False,
+                    'is_script': False, 'licenses': [], 'license_expressions': [],
+                    'percentage_of_license_text': 0, 'copyrights': [], 'holders': [],
+                    'authors': [], 'packages': [], 'emails': [], 'urls': [], 'files_count': 9,
+                    'dirs_count': 1, 'size_count': 32826, 'scan_errors': []}
+
+        # We will only check the first element in the inventory list 
+        assert inventory[0] == expected
+
+
     def test_generation_dir_endswith_space(self):
         location = get_test_loc('test_gen/inventory/complex/about_file_path_dir_endswith_space.csv')
         base_dir = get_temp_dir()
@@ -141,7 +207,7 @@ custom1: |
         location = get_test_loc('test_gen/inv2.csv')
         base_dir = get_temp_dir()
         errors, abouts = gen.generate(location, base_dir)
-        expected = OrderedDict([('.', None)])
+        expected = dict([('.', None)])
         assert abouts[0].about_resource.value == expected
         assert len(errors) == 1
 
@@ -150,7 +216,7 @@ custom1: |
         base_dir = get_temp_dir()
 
         errors, abouts = gen.generate(location, base_dir)
-        expected = OrderedDict([('test.tar.gz', None)])
+        expected = dict([('test.tar.gz', None)])
 
         assert abouts[0].about_resource.value == expected
         assert len(errors) == 1
@@ -162,7 +228,7 @@ custom1: |
         base_dir = get_temp_dir()
 
         errors, abouts = gen.generate(location, base_dir)
-        expected = OrderedDict([('test.tar.gz', None)])
+        expected = dict([('test.tar.gz', None)])
 
         assert abouts[0].about_resource.value == expected
         assert len(errors) == 1
@@ -172,11 +238,12 @@ custom1: |
         base_dir = get_temp_dir()
 
         errors, abouts = gen.generate(location, base_dir)
-        msg1 = 'Field custom1 is a custom field.'
-        msg2 = 'Field about_resource'
+        err_msg_list = []
+        for severity, message in errors:
+            err_msg_list.append(message)
+        msg1 = "Field ['custom1'] is a custom field."
 
-        assert msg1 in errors[0].message
-        assert msg2 in errors[1].message
+        assert msg1 in err_msg_list
 
         result = [a.dumps() for a in abouts][0]
         expected = (
@@ -206,10 +273,13 @@ name: test
 version: '1.5'
 licenses:
   - key: License1
+    name: License1
     file: LIC1.LICENSE
   - key: License2
+    name: License2
     file: LIC2.LICENSE
   - key: License3
+    name: License3
     file: LIC3.LICENSE
 '''
         )
@@ -227,11 +297,87 @@ licenses:
 name: test.c
 licenses:
   - key: License1
+    name: License1
     file: LIC1.LICENSE, LIC2.LICENSE
 '''
         )
         assert expected == result
 
+    def test_generate_license_key_with_custom_file_450_no_fetch(self):
+        location = get_test_loc('test_gen/lic_issue_450/custom_and_valid_lic_key_with_file.csv')
+        base_dir = get_temp_dir()
+
+        errors, abouts = gen.generate(location, base_dir)
+
+        result = [a.dumps() for a in abouts][0]
+        expected = (
+'''about_resource: test.c
+name: test.c
+license_expression: mit AND custom
+licenses:
+  - file: custom.txt
+'''
+        )
+        assert expected == result
+
+
+    def test_generate_license_key_with_custom_file_450_with_fetch_with_order(self):
+        location = get_test_loc('test_gen/lic_issue_450/custom_and_valid_lic_key_with_file.csv')
+        base_dir = get_temp_dir()
+
+        errors, abouts = gen.generate(location, base_dir)
+
+        lic_dict = {u'mit': [u'MIT License',
+                                       u'mit.LICENSE',
+                                       u'This component is released under MIT License.',
+                                       u'https://enterprise.dejacode.com/urn/?urn=urn:dje:license:mit',
+                                       u'mit'
+                                       ]}
+        # The first row from the test file
+        a = abouts[0]
+        a.license_key.value.append('mit')
+        a.license_key.value.append('custom')
+        result1 = a.dumps(lic_dict)
+        # The second row from the test file
+        b = abouts[1]
+        b.license_key.value.append('custom')
+        b.license_key.value.append('mit')
+        result2 = b.dumps(lic_dict)
+
+        expected1 = (
+'''about_resource: test.c
+name: test.c
+license_expression: mit AND custom
+licenses:
+  - key: mit
+    name: MIT License
+    file: mit.LICENSE
+    url: https://enterprise.dejacode.com/urn/?urn=urn:dje:license:mit
+    spdx_license_key: mit
+  - key: custom
+    name: custom
+    file: custom.txt
+'''
+        )
+
+        expected2 = (
+'''about_resource: test.h
+name: test.h
+license_expression: custom AND mit
+licenses:
+  - key: custom
+    name: custom
+    file: custom.txt
+  - key: mit
+    name: MIT License
+    file: mit.LICENSE
+    url: https://enterprise.dejacode.com/urn/?urn=urn:dje:license:mit
+    spdx_license_key: mit
+'''
+        )
+        assert expected1 == result1
+        assert expected2 == result2
+
     @skip('FIXME: this test is making a failed, live API call')
     def test_generate_not_overwrite_original_license_file(self):
         location = get_test_loc('test_gen/inv5.csv')
@@ -264,4 +410,4 @@ licenses:
                     u'redistribute: yes\n'
                     u'attribute: yes\n'
                     u'modified: no\n')
-        assert expected == in_mem_result
\ No newline at end of file
+        assert expected == in_mem_result
